﻿/*
Documents Database Migration Pre-Check Script
--------------------------------------------------------------------------------------
 This file contains SQL statements to detect data inconsistencies before 
 migrate to 7.8 or upper version.        
--------------------------------------------------------------------------------------
*/
WITH BrokenBinaryData(FileId)  
AS  
(  
    SELECT [FileId]
	FROM [dbo].[BinaryData]
	WHERE [SqlPath] in (SELECT [SqlPath] FROM [dbo].[BinaryData] GROUP BY [SqlPath] HAVING COUNT(*) > 1)   
), 
BrokenExtension (ExtensionId)
AS (
	SELECT [ExtensionId]
	FROM [dbo].[Extension] WHERE [FileId] IN (SELECT [FileId] FROM BrokenBinaryData)
),
ExtensionLostBinary (ExtensionId)
AS (
	SELECT [ExtensionId]
	FROM [dbo].[Extension] WHERE [FileId] IS NOT NULL AND NOT EXISTS (SELECT 1 FROM [dbo].[BinaryData] WHERE [BinaryData].[FileId] = [Extension].[FileId])
)
SELECT DISTINCT [Document].[RootId] as [Id], 'One or more document binaries refer to the same object' as [Description]
FROM [dbo].[Document] INNER JOIN BrokenBinaryData ON [Document].[FileId] = BrokenBinaryData.FileId
UNION ALL
SELECT DISTINCT [Document].[RootId], 'One or more page binaries refer to the same object'
FROM [dbo].[Document] INNER JOIN [dbo].[Page] ON [Document].[Id] = [Page].[DocumentId]
					  INNER JOIN BrokenBinaryData ON [Page].[FileId] = BrokenBinaryData.FileId
UNION ALL
SELECT DISTINCT [Document].[RootId], 'One or more document binary extensions refer to the same object'
FROM [dbo].[Document] INNER JOIN [dbo].[DocumentExtension] ON [Document].[Id] = [DocumentExtension].[DocumentId]
					  INNER JOIN BrokenExtension ON  [DocumentExtension].[ExtensionId] = BrokenExtension.ExtensionId
UNION ALL
SELECT DISTINCT [Document].[RootId], 'One or more page binary extensions refer to the same object'
FROM [dbo].[Document] INNER JOIN [dbo].[Page] ON [Page].[DocumentId] = [Document].[Id]
					  INNER JOIN [dbo].[PageExtension] ON [Page].[PageId] = [PageExtension].[PageId]
					  INNER JOIN BrokenExtension ON [PageExtension].[ExtensionId] = BrokenExtension.ExtensionId
UNION ALL
SELECT DISTINCT CASE WHEN [Folder].[RootId] IS NULL THEN [Folder].[Id] ELSE [Folder].[RootId] END, 
	   	   	    'One or more folder binary extensions refer to the same object'
FROM [dbo].[Folder] INNER JOIN [dbo].[FolderExtension] ON [Folder].[Id] = [FolderExtension].[FolderId]
					INNER JOIN BrokenExtension ON  [FolderExtension].[ExtensionId] = BrokenExtension.ExtensionId
UNION ALL
SELECT DISTINCT [Document].[RootId], 'One or more document binaries are referenced multiple times'
FROM [dbo].[Document] WHERE [FileId] IN (SELECT [FileId] FROM [dbo].[Document] GROUP BY FileId HAVING COUNT(*) > 1)
UNION ALL
SELECT DISTINCT [Document].[RootId], 'Several table rows duplicate each other'
FROM [dbo].[Document] INNER JOIN [dbo].[DocumentField] ON [Document].[Id] = [DocumentField].[DocumentId]
WHERE [DocumentField].[FieldId] in (
       SELECT FieldId 
       FROM [dbo].[TableRow]
       GROUP BY FieldId, Id
       HAVING  Count(Id) > 1)
UNION ALL
SELECT DISTINCT CASE WHEN [Folder].[RootId] IS NULL THEN [Folder].[Id] ELSE [Folder].[RootId] END,
				'One or more folder binary extensions refer to not existing binary data'
FROM [dbo].[Folder] INNER JOIN [dbo].[FolderExtension] ON [Folder].[Id] = [FolderExtension].[FolderId]
					INNER JOIN ExtensionLostBinary ON [FolderExtension].[ExtensionId] = ExtensionLostBinary.ExtensionId
UNION ALL
SELECT DISTINCT [RootId], 'One or more document binary extensions refer to not exisitng binary data' 
FROM [dbo].[Document] INNER JOIN [dbo].[DocumentExtension] ON [Document].[Id] = [DocumentExtension].[DocumentId]
					  INNER JOIN ExtensionLostBinary ON [DocumentExtension].[ExtensionId] = ExtensionLostBinary.ExtensionId
UNION ALL
SELECT DISTINCT [RootId], 'One or more page binary extensions refer to not exisitng binary data'
FROM [dbo].[Document] INNER JOIN [dbo].[Page] ON [Document].[Id] = [Page].[DocumentId]
					  INNER JOIN [dbo].[PageExtension] ON [Page].[PageId] = [PageExtension].[PageId]
					  INNER JOIN ExtensionLostBinary ON [PageExtension].[ExtensionId] = ExtensionLostBinary.ExtensionId	
UNION ALL
SELECT DISTINCT [RootId], 'One or more document source files do not exist'
FROM [dbo].[Document] WHERE [FileId] IS NOT NULL AND NOT EXISTS (SELECT 1 FROM [dbo].[BinaryData] WHERE [FileId] = [Document].[FileId])
UNION ALL
SELECT DISTINCT [RootId], 'One or more page source files do not exist'
FROM [dbo].[Document] WHERE [Id] IN 
	(SELECT [DocumentId] FROM [dbo].[Page] 
	 WHERE [FileId] IS NOT NULL AND NOT EXISTS (SELECT 1 FROM [dbo].[BinaryData] WHERE [FileId] = [Page].[FileId]))
UNION ALL
SELECT DISTINCT [RootId], 'One or more page textlines refer to not exisitng binary data'
FROM [dbo].[Document] WHERE [Id] IN 
	(SELECT [DocumentId] FROM [dbo].[Page] 
	 WHERE [TextlinesId] IS NOT NULL AND NOT EXISTS (SELECT 1 FROM [dbo].[BinaryData] WHERE [FileId] = [Page].[TextlinesId]))
UNION ALL
SELECT DISTINCT [RootId], 'One or more pages have duplicated extensions'  
FROM [dbo].[Document] d INNER JOIN [dbo].[Page] p on d.[Id] = p.[DocumentId]
WHERE p.[PageId] IN 
(
	SELECT pe.[PageId]
	FROM [dbo].[PageExtension] pe  
	INNER JOIN [dbo].[Extension] e ON e.[ExtensionId] = pe.[ExtensionId] 				
	GROUP BY pe.[PageId], e.[Name] 
	HAVING (COUNT(e.[Name]) > 1)
)
UNION ALL
SELECT DISTINCT [RootId], 'One or more documents have duplicated extensions'  
FROM [dbo].[Document]
WHERE [Id] IN 
(
	SELECT de.[DocumentId]
	FROM [dbo].[DocumentExtension] de  
	INNER JOIN [dbo].[Extension] e ON e.[ExtensionId] = de.[ExtensionId] 				
	GROUP BY de.[DocumentId], e.[Name] 
	HAVING (COUNT(e.[Name]) > 1)
)
UNION ALL
SELECT DISTINCT CASE WHEN [RootId] IS NULL THEN [Id] ELSE [RootId] END,
	'One or more folders have duplicated extensions'  
FROM [dbo].[Folder] 
WHERE [Id] IN 
(
	SELECT fe.[FolderId]
	FROM [dbo].[FolderExtension] fe  
	INNER JOIN [dbo].[Extension] e ON e.[ExtensionId] = fe.[ExtensionId] 				
	GROUP BY fe.[FolderId], e.[Name] 
	HAVING (COUNT(e.[Name]) > 1)
)
UNION ALL
SELECT DISTINCT [RootId], 'One or more pages have unexpectedly large size of barcodes data'
FROM [dbo].[Document] d INNER JOIN [dbo].[Page] p on d.[Id] = p.[DocumentId]
WHERE DATALENGTH(p.[Barcodes]) > 10485760 /* This is 5Mb .NET string which can hold ~25k of barcodes in XML format. More than enough. */
UNION ALL
SELECT DISTINCT doc.[RootId], 'Root folder association is broken for some documents'
FROM [dbo].[Document] doc
WHERE NOT EXISTS
(SELECT 1 FROM [dbo].[ObjectHierarchy] oh WHERE oh.[ChildId] = doc.[Id] AND oh.[ParentId] = doc.[RootId])